# vim: ts=4:sts=4:sw=4
#
# @author: <sylvain.herledan@oceandatalab.com>
# @date: 2016-09-06
#
# This file is part of SEAScope, a 3D visualisation and analysis application
# for satellite, in-situ and numerical model data.
#
# Copyright (C) 2014-2023 OceanDataLab
#
# SEAScope is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with SEAScope. If not, see <https://www.gnu.org/licenses/>.

"""
This module handles the serialization of the addVariable command
"""

import logging
import flatbuffers
import SEAScope.API.OpCode
import SEAScope.API.CommandArgs
import SEAScope.API.AddVariable
import SEAScope.API.Command
import SEAScope.API.BasicResponse

import SEAScope.types.renderable_id
import SEAScope.types.variable

logger = logging.getLogger(__name__)


def serialize(builder, target, var_obj):
    """
    Serialize a addVariable command using FlatBuffers.

    Parameters
    ----------
    builder : flatbuffers.builder.Builder
        The FlatBuffers builder instance which serializes data. If this
        parameter is None, then a new builder will be created
    target : dict
        The identifier that must be used for the new variable. The dict must
        satisfy the requirements of the
        :func:`SEAScope.types.renderable_id.serialize` method
    var_obj : dict
        The parameters of the variable to add. The dict must satisfy the
        requirements of the
        :func:`SEAScope.types.variable.serialize` method

    Returns
    -------
    tuple(flatbuffers.builder.Builder, int)
        A tuple which contains two elements:

        - the :obj:`flatbuffers.builder.Builder` instance which has been used
          to serialize data
        - an :obj:`int` which is the address/offset of the serialized object
          in the builder buffer
    """
    if builder is None:
        builder = flatbuffers.Builder(0)

    _, _variable = SEAScope.types.variable.serialize(builder, var_obj)
    _, _rid = SEAScope.types.renderable_id.serialize(builder, target)

    cls = SEAScope.API.AddVariable
    cls.AddVariableStart(builder)
    cls.AddVariableAddCollectionId(builder, _rid)
    cls.AddVariableAddVariable(builder, _variable)
    cmd_args = cls.AddVariableEnd(builder)

    opcode = SEAScope.API.OpCode.OpCode().addVariable
    args_type = SEAScope.API.CommandArgs.CommandArgs().AddVariable
    SEAScope.API.Command.CommandStart(builder)
    SEAScope.API.Command.CommandAddOpcode(builder, opcode)
    SEAScope.API.Command.CommandAddArgsType(builder, args_type)
    SEAScope.API.Command.CommandAddArgs(builder, cmd_args)
    cmd = SEAScope.API.Command.CommandEnd(builder)

    return builder, cmd


def deserialize(buf):
    """
    Deserialize response sent by SEAScope after executing the addVariable
    command.

    Parameters
    ----------
    buf : bytearray
        The buffer which contains the result of the addVariable command
        serialized with FlatBuffers

    Returns
    -------
    dict
        The result of the addVariable command. The dict contains two keys:

        - ``ok`` : a :obj:`bool` representing the success (True) or failure
          (False) of the command execution

        - ``msg`` : a :obj:`str` providing details in case of failure
    """
    cls = SEAScope.API.BasicResponse.BasicResponse
    res = cls.GetRootAsBasicResponse(buf, 0)
    return {'ok': res.Success(), 'msg': res.Message().decode('utf-8')}



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: API

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class Source(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsSource(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Source()
        x.Init(buf, n + offset)
        return x

    # Source
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Source
    def Id(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, o + self._tab.Pos)
        return 0

    # Source
    def Title(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Source
    def Description(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Source
    def Tags(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from SEAScope.API.Tag import Tag
            obj = Tag()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Source
    def TagsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Source
    def TagsIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        return o == 0

def SourceStart(builder): builder.StartObject(4)
def SourceAddId(builder, id): builder.PrependUint32Slot(0, id, 0)
def SourceAddTitle(builder, title): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(title), 0)
def SourceAddDescription(builder, description): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(description), 0)
def SourceAddTags(builder, tags): builder.PrependUOffsetTRelativeSlot(3, flatbuffers.number_types.UOffsetTFlags.py_type(tags), 0)
def SourceStartTagsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def SourceEnd(builder): return builder.EndObject()



# vim: ts=4:sts=4:sw=4
#
# @author: <sylvain.herledan@oceandatalab.com>
# @date: 2024-08-30
#
# This file is part of SEAScope, a 3D visualisation and analysis application
# for satellite, in-situ and numerical model data.
#
# Copyright (C) 2014-2024 OceanDataLab
#
# SEAScope is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
"""
This module is used a very limited mock for the ``imp`` module which does not
exist on Python >= 3.12 anymore but is used by flatbuffers v1.x.
"""
import importlib

find_module = importlib.__import__



import SEAScope.upload

with SEAScope.upload.connect('localhost', 11155) as link:
    SEAScope.upload.select_variable_by_id(link, 2, 1, 1, True, False)

    SEAScope.upload.select_variable_by_label(link, 'Sentinel-3 OLCI', 'Oa09e',
                                             True, True)
